from django.db import models

from coderedcms.widgets import ColorPickerWidget
from django.forms.widgets import Textarea


class ColorField(models.CharField):
    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 255
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs['widget'] = ColorPickerWidget
        return super().formfield(**kwargs)


class MonospaceField(models.TextField):
    def formfield(self, **kwargs):
        kwargs["widget"] = Textarea(attrs={
            "rows": 12,
            "class": "monospace",
            "spellcheck": "false",
        })
        return super().formfield(**kwargs)



# Generated by Django 3.1.7 on 2021-05-27 14:30

from django.db import migrations
import wagtail.core.blocks
import wagtail.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('coderedcms', '0019_spelling_corrections'),
    ]

    operations = [
        migrations.AlterField(
            model_name='coderedpage',
            name='struct_org_hours',
            field=wagtail.core.fields.StreamField([('hours', wagtail.core.blocks.StructBlock([('days', wagtail.core.blocks.MultipleChoiceBlock(choices=[('Monday', 'Monday'), ('Tuesday', 'Tuesday'), ('Wednesday', 'Wednesday'), ('Thursday', 'Thursday'), ('Friday', 'Friday'), ('Saturday', 'Saturday'), ('Sunday', 'Sunday')], help_text='For late night hours past 23:59, define each day in a separate block.', verbose_name='Days')), ('start_time', wagtail.core.blocks.TimeBlock(verbose_name='Opening time')), ('end_time', wagtail.core.blocks.TimeBlock(verbose_name='Closing time'))]))], blank=True, verbose_name='Hours of operation'),
        ),
    ]



# Generated by Django 2.1.5 on 2019-03-20 02:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('coderedcms', '0009_auto_20190201_1546'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='generalsettings',
            name='robots',
        ),
    ]



from .base import *  # noqa

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = False

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '{{ secret_key }}'

# Add your site's domain name(s) here.
ALLOWED_HOSTS = ['{{ domain }}']

# To send email from the server, we recommend django_sendmail_backend
# Or specify your own email backend such as an SMTP server.
# https://docs.djangoproject.com/en/{{ docs_version }}/ref/settings/#email-backend
EMAIL_BACKEND = 'django_sendmail_backend.backends.EmailBackend'

# Default email address used to send messages from the website.
DEFAULT_FROM_EMAIL = '{{ sitename }} <info@{{ domain_nowww }}>'

# A list of people who get error notifications.
ADMINS = [
    ('Administrator', 'admin@{{ domain_nowww }}'),
]

# A list in the same format as ADMINS that specifies who should get broken link
# (404) notifications when BrokenLinkEmailsMiddleware is enabled.
MANAGERS = ADMINS

# Email address used to send error messages to ADMINS.
SERVER_EMAIL = DEFAULT_FROM_EMAIL

# DATABASES = {
#     'default': {
#         'ENGINE': 'django.db.backends.mysql',
#         'HOST': 'localhost',
#         'NAME': '{{ project_name }}',
#         'USER': '{{ project_name }}',
#         'PASSWORD': '',
#     }
# }

# Use template caching to speed up wagtail admin and front-end.
# Requires reloading web server to pick up template changes.
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'wagtail.contrib.settings.context_processors.settings',
            ],
            'loaders': [
                ('django.template.loaders.cached.Loader', [
                    'django.template.loaders.filesystem.Loader',
                    'django.template.loaders.app_directories.Loader',
                ]),
            ],
        },
    },
]

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.filebased.FileBasedCache',
        'LOCATION': os.path.join(BASE_DIR, 'cache'),  # noqa
        'KEY_PREFIX': 'coderedcms',
        'TIMEOUT': 14400,  # in seconds
    }
}

try:
    from .local_settings import *  # noqa
except ImportError:
    pass



"""
Models module entry point. Used to cleanly organize various models
into files based on their purpose, but provide them all via
a single `models` module.
"""

from .integration_models import *  # noqa
from .page_models import *  # noqa
from .snippet_models import *  # noqa
from .wagtailsettings_models import *  # noqa



import os
import shutil
import sys
import unittest
from coderedcms.bin.coderedcms import main as coderedcms_main


class TestCoderedcmsStart(unittest.TestCase):
    CURR_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    TEST_DIR = os.path.join(CURR_DIR, 'testproject-unittest')

    def setup(self):
        # Clean/create directory to start into
        if os.path.exists(self.TEST_DIR):
            shutil.rmtree(self.TEST_DIR)
        os.mkdir(self.TEST_DIR)

    def cleanup(self):
        # Cleanup
        if os.path.exists(self.TEST_DIR):
            shutil.rmtree(self.TEST_DIR)

    def test_help(self):
        # Set args
        sys.argv = ['coderedcms', 'help']
        # Run
        coderedcms_main()
        # Nothing to assert here... just make sure it doesn't error out.

    def test_help_start(self):
        # Set args
        sys.argv = ['coderedcms', 'help', 'start']
        # Run
        coderedcms_main()
        # Nothing to assert here... just make sure it doesn't error out.

    def test_default(self):
        self.setup()
        # Set args
        sys.argv = ['coderedcms', 'start', 'myproject', self.TEST_DIR]
        # Run
        coderedcms_main()
        # Assert files exist
        self.assertTrue(os.path.exists(os.path.join(self.TEST_DIR, 'README.md')))
        self.cleanup()

    def test_allopts(self):
        self.setup()
        # Set args
        sys.argv = [
            'coderedcms',
            'start',
            'myproject',
            self.TEST_DIR,
            '--template', 'basic',
            '--sitename', 'MegaCorp, Inc.',
            '--domain', 'example.com'
        ]
        # Run
        coderedcms_main()
        # Assert files exist
        self.assertTrue(os.path.exists(os.path.join(self.TEST_DIR, 'README.md')))
        self.cleanup()

    def test_domain_www(self):
        self.setup()
        # Set args
        sys.argv = [
            'coderedcms',
            'start',
            'myproject',
            self.TEST_DIR,
            '--domain', 'www.example.com'
        ]
        # Run
        coderedcms_main()
        # Assert files exist
        self.assertTrue(os.path.exists(os.path.join(self.TEST_DIR, 'README.md')))
        self.cleanup()

    def test_template_sass(self):
        self.setup()
        # Set args
        sys.argv = ['coderedcms', 'start', 'myproject', self.TEST_DIR, '--template', 'sass']
        # Run
        coderedcms_main()
        # Assert files exist
        self.assertTrue(os.path.exists(os.path.join(self.TEST_DIR, 'README.md')))
        self.cleanup()



from django.conf import settings
from django.urls import include, path, re_path
from django.contrib import admin
from wagtail.documents import urls as wagtaildocs_urls
from coderedcms import admin_urls as coderedadmin_urls
from coderedcms import search_urls as coderedsearch_urls
from coderedcms import urls as codered_urls

urlpatterns = [
    # Admin
    path('django-admin/', admin.site.urls),
    path('admin/', include(coderedadmin_urls)),

    # Documents
    path('docs/', include(wagtaildocs_urls)),

    # Search
    path('search/', include(coderedsearch_urls)),

    # For anything not caught by a more specific rule above, hand over to
    # the page serving mechanism. This should be the last pattern in
    # the list:
    re_path(r'', include(codered_urls)),

    # Alternatively, if you want CMS pages to be served from a subpath
    # of your site, rather than the site root:
    #    re_path(r"^pages/", include(codered_urls)),
]


if settings.DEBUG:
    from django.conf.urls.static import static
    from django.contrib.staticfiles.urls import staticfiles_urlpatterns

    # Serve static and media files from development server
    urlpatterns += staticfiles_urlpatterns()
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



import mimetypes

from django.contrib.contenttypes.models import ContentType
from django.templatetags.static import static
from django.http.response import HttpResponse
from django.urls import reverse
from django.utils.html import format_html, mark_safe
from django.utils.translation import gettext_lazy as _
from wagtail.core import hooks
from wagtail.core.models import UserPagePermissionsProxy, get_page_models
from wagtailcache.cache import clear_cache

from coderedcms.wagtail_flexible_forms.wagtail_hooks import FormAdmin, SubmissionAdmin


@hooks.register('insert_global_admin_css')
def global_admin_css():
    return format_html(
        '<link rel="stylesheet" type="text/css" href="{}">',
        static('coderedcms/css/codered-admin.css')
    )


@hooks.register('insert_editor_css')
def editor_css():
    return format_html(
        '<link rel="stylesheet" type="text/css" href="{}">',
        static('coderedcms/css/codered-editor.css')
    )


@hooks.register('insert_editor_js')
def collapsible_js():
    return format_html('<script src="{}"></script>', static('coderedcms/js/codered-editor.js'))


@hooks.register('after_create_page')
@hooks.register('after_edit_page')
def clear_wagtailcache(request, page):
    if page.live:
        clear_cache()


@hooks.register('filter_form_submissions_for_user')
def codered_forms(user, editable_forms):
    from coderedcms.models import CoderedFormMixin
    """
    Add our own CoderedFormPage to editable_forms, since wagtail is unaware
    of its existence. Essentially this is a fork of wagtail.contrib.forms.get_forms_for_user()
    and wagtail.contrib.forms.get_form_types()
    """
    form_models = [
        model for model in get_page_models()
        if issubclass(model, CoderedFormMixin)
    ]
    form_types = list(
        ContentType.objects.get_for_models(*form_models).values()
    )

    editable_forms = UserPagePermissionsProxy(user).editable_pages()
    editable_forms = editable_forms.filter(content_type__in=form_types)

    return editable_forms


@hooks.register('before_serve_document')
def serve_document_directly(document, request):
    """
    This hook prevents documents from being downloaded unless
    specified by an <a> tag with the download attribute.
    """
    content_type, content_encoding = mimetypes.guess_type(document.filename)
    response = HttpResponse(document.file.read(), content_type=content_type)
    response['Content-Disposition'] = 'inline;filename="{0}"'.format(document.filename)
    response['Content-Encoding'] = content_encoding
    return response


class CoderedSubmissionAdmin(SubmissionAdmin):

    def __init__(self, parent=None):
        from coderedcms.models import CoderedSessionFormSubmission
        self.model = CoderedSessionFormSubmission
        super().__init__(parent=parent)


class CoderedFormAdmin(FormAdmin):
    list_display = ('title', 'action_links')

    def all_submissions_link(self, obj, label=_('See all submissions'),
                             url_suffix=''):
        return '<a href="%s?page_id=%s%s">%s</a>' % (
            reverse(CoderedSubmissionAdmin().url_helper.get_action_url_name('index')),
            obj.pk, url_suffix, label)
    all_submissions_link.short_description = ''
    all_submissions_link.allow_tags = True

    def action_links(self, obj):
        from coderedcms.models import CoderedFormPage, CoderedStreamFormPage
        actions = []
        if issubclass(type(obj.specific), CoderedFormPage):
            actions.append(
                '<a href="{0}">{1}</a>'.format(reverse(
                    'wagtailforms:list_submissions',
                    args=(obj.pk,)),
                    _('See all Submissions')
                )
            )
            actions.append(
                '<a href="{0}">{1}</a>'.format(
                    reverse("wagtailadmin_pages:edit", args=(obj.pk,)), _("Edit this form page")
                )
            )
        elif issubclass(type(obj.specific), CoderedStreamFormPage):
            actions.append(self.unprocessed_submissions_link(obj))
            actions.append(self.all_submissions_link(obj))
            actions.append(self.edit_link(obj))

        return mark_safe("<br />".join(actions))

# modeladmin_register(CoderedFormAdmin)
# modeladmin_register(CoderedSubmissionAdmin)



from django import forms
from django.db.models import BLANK_CHOICE_DASH
from django.utils.dateparse import parse_datetime
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _
from anyascii import anyascii
from wagtail.core.blocks import (
    StructBlock, TextBlock, CharBlock, BooleanBlock, ListBlock, StreamBlock,
    DateBlock, TimeBlock, DateTimeBlock, ChoiceBlock, RichTextBlock,
)


class FormFieldBlock(StructBlock):
    field_label = CharBlock(label=_('Label'))
    help_text = TextBlock(required=False, label=_('Help text'))

    field_class = forms.CharField
    widget = None

    def get_slug(self, struct_value):
        return slugify(anyascii(struct_value['field_label']))

    def get_field_class(self, struct_value):
        return self.field_class

    def get_widget(self, struct_value):
        return self.widget

    def get_field_kwargs(self, struct_value):
        kwargs = {'label': struct_value['field_label'],
                  'help_text': struct_value['help_text'],
                  'required': struct_value.get('required', False)}
        if 'default_value' in struct_value:
            kwargs['initial'] = struct_value['default_value']
        form_widget = self.get_widget(struct_value)
        if form_widget is not None:
            kwargs['widget'] = form_widget
        return kwargs

    def get_field(self, struct_value):
        return self.get_field_class(struct_value)(
            **self.get_field_kwargs(struct_value))


class OptionalFormFieldBlock(FormFieldBlock):
    required = BooleanBlock(label=_('Required'), required=False)


CHARFIELD_FORMATS = [
    ('email', _('Email')),
    ('url', _('URL')),
]
try:
    from phonenumber_field.formfields import PhoneNumberField
except ImportError:
    pass
else:
    CHARFIELD_FORMATS.append(('phone', _('Phone')))


class CharFieldBlock(OptionalFormFieldBlock):
    format = ChoiceBlock(choices=CHARFIELD_FORMATS, required=False, label=_('Format'))
    default_value = CharBlock(required=False, label=_('Default value'))

    class Meta:
        label = _('Text field (single line)')

    def get_field_class(self, struct_value):
        text_format = struct_value['format']
        if text_format == 'url':
            return forms.URLField
        if text_format == 'email':
            return forms.EmailField
        if text_format == 'phone':
            return PhoneNumberField
        return super().get_field_class(struct_value)


class TextFieldBlock(OptionalFormFieldBlock):
    default_value = TextBlock(required=False, label=_('Default value'))

    widget = forms.Textarea(attrs={'rows': 5})

    class Meta:
        label = _('Text field (multi line)')


class NumberFieldBlock(OptionalFormFieldBlock):
    default_value = CharBlock(required=False, label=_('Default value'))

    widget = forms.NumberInput

    class Meta:
        label = _('Number field')


class CheckboxFieldBlock(FormFieldBlock):
    default_value = BooleanBlock(required=False)

    field_class = forms.BooleanField

    class Meta:
        label = _('Checkbox field')
        icon = 'tick-inverse'


class RadioButtonsFieldBlock(OptionalFormFieldBlock):
    choices = ListBlock(CharBlock(label=_('Choice')))

    field_class = forms.ChoiceField
    widget = forms.RadioSelect

    class Meta:
        label = _('Radio buttons')
        icon = 'radio-empty'

    def get_field_kwargs(self, struct_value):
        kwargs = super().get_field_kwargs(struct_value)
        kwargs['choices'] = [(choice, choice)
                             for choice in struct_value['choices']]
        return kwargs


class DropdownFieldBlock(RadioButtonsFieldBlock):
    widget = forms.Select

    class Meta:
        label = _('Dropdown field')
        icon = 'arrow-down-big'

    def get_field_kwargs(self, struct_value):
        kwargs = super(DropdownFieldBlock,
                       self).get_field_kwargs(struct_value)
        kwargs['choices'].insert(0, BLANK_CHOICE_DASH[0])
        return kwargs


class CheckboxesFieldBlock(OptionalFormFieldBlock):
    checkboxes = ListBlock(CharBlock(label=_('Checkbox')))

    field_class = forms.MultipleChoiceField
    widget = forms.CheckboxSelectMultiple

    class Meta:
        label = _('Multiple checkboxes field')
        icon = 'list-ul'

    def get_field_kwargs(self, struct_value):
        kwargs = super(CheckboxesFieldBlock,
                       self).get_field_kwargs(struct_value)
        kwargs['choices'] = [(choice, choice)
                             for choice in struct_value['checkboxes']]
        return kwargs


class DatePickerInput(forms.DateInput):
    def __init__(self, *args, **kwargs):
        attrs = kwargs.get('attrs')
        if attrs is None:
            attrs = {}
        attrs.update({
            'data-provide': 'datepicker',
            'data-date-format': 'yyyy-mm-dd',
        })
        kwargs['attrs'] = attrs
        super().__init__(*args, **kwargs)


class DateFieldBlock(OptionalFormFieldBlock):
    default_value = DateBlock(required=False)

    field_class = forms.DateField
    widget = DatePickerInput

    class Meta:
        label = _('Date field')
        icon = 'date'


class HTML5TimeInput(forms.TimeInput):
    input_type = 'time'


class TimeFieldBlock(OptionalFormFieldBlock):
    default_value = TimeBlock(required=False)

    field_class = forms.TimeField
    widget = HTML5TimeInput

    class Meta:
        label = _('Time field')
        icon = 'time'


class DateTimePickerInput(forms.SplitDateTimeWidget):
    def __init__(self, attrs=None, date_format=None, time_format=None):
        super().__init__(attrs=attrs,
                         date_format=date_format, time_format=time_format)
        self.widgets = (
            DatePickerInput(attrs=attrs, format=date_format),
            HTML5TimeInput(attrs=attrs, format=time_format),
        )

    def decompress(self, value):
        if isinstance(value, str):
            value = parse_datetime(value)
        return super().decompress(value)


class DateTimeFieldBlock(OptionalFormFieldBlock):
    default_value = DateTimeBlock(required=False)

    field_class = forms.SplitDateTimeField
    widget = DateTimePickerInput

    class Meta:
        label = _('Date+time field')
        icon = 'date'


class ImageFieldBlock(OptionalFormFieldBlock):
    field_class = forms.ImageField

    class Meta:
        label = _('Image field')
        icon = 'image'


class FileFieldBlock(OptionalFormFieldBlock):
    field_class = forms.FileField

    class Meta:
        label = _('File field')
        icon = 'download'


class FormFieldsBlock(StreamBlock):
    char = CharFieldBlock(group=_('Fields'))
    text = TextFieldBlock(group=_('Fields'))
    number = NumberFieldBlock(group=_('Fields'))
    checkbox = CheckboxFieldBlock(group=_('Fields'))
    radios = RadioButtonsFieldBlock(group=_('Fields'))
    dropdown = DropdownFieldBlock(group=_('Fields'))
    checkboxes = CheckboxesFieldBlock(group=_('Fields'))
    date = DateFieldBlock(group=_('Fields'))
    time = TimeFieldBlock(group=_('Fields'))
    datetime = DateTimeFieldBlock(group=_('Fields'))
    image = ImageFieldBlock(group=_('Fields'))
    file = FileFieldBlock(group=_('Fields'))
    text_markup = RichTextBlock(group=_('Other'))

    class Meta:
        label = _('Form fields')


class FormStepBlock(StructBlock):
    name = CharBlock(label=_('Name'), required=False)
    form_fields = FormFieldsBlock()

    class Meta:
        label = _('Form step')


class FormStepsBlock(StreamBlock):
    step = FormStepBlock()

    class Meta:
        label = _('Form steps')



from django.conf.urls import url
from django.contrib.admin import SimpleListFilter
from django.contrib.admin.utils import quote
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from wagtail.contrib.modeladmin.helpers import (
    PermissionHelper, PagePermissionHelper, PageAdminURLHelper, AdminURLHelper,
    ButtonHelper)
from wagtail.contrib.modeladmin.options import ModelAdmin
from wagtail.contrib.modeladmin.views import IndexView, InstanceSpecificView
from wagtail.admin import messages
from wagtail.core import hooks
from wagtail.core.models import Page
from wagtail.contrib.forms.utils import get_forms_for_user

from .models import SessionFormSubmission


class FormIndexView(IndexView):
    page_title = _('Forms')


class FormPermissionHelper(PagePermissionHelper):
    def user_can_list(self, user):
        return get_forms_for_user(user).exists()

    def user_can_create(self, user):
        return False

    def user_can_edit_obj(self, user, obj):
        return False

    def user_can_delete_obj(self, user, obj):
        return False

    def user_can_publish_obj(self, user, obj):
        return False

    def user_can_unpublish_obj(self, user, obj):
        return False

    def user_can_copy_obj(self, user, obj):
        return False

    def user_can_inspect_obj(self, user, obj):
        return False


class FormURLHelper(PageAdminURLHelper):
    def _get_action_url_pattern(self, action):
        if action == 'index':
            return r'^stream_forms/$'
        return r'^stream_forms/%s/$' % action


class FormAdmin(ModelAdmin):
    model = Page
    menu_label = _('Forms')
    menu_icon = 'form'
    list_display = ('title', 'unprocessed_submissions_link',
                    'all_submissions_link', 'edit_link')
    index_view_class = FormIndexView
    permission_helper_class = FormPermissionHelper
    url_helper_class = FormURLHelper

    def get_queryset(self, request):
        return get_forms_for_user(request.user)

    def all_submissions_link(self, obj, label=_('See all submissions'),
                             url_suffix=''):
        return '<a href="%s?page_id=%s%s">%s</a>' % (
            reverse(SubmissionAdmin().url_helper.get_action_url_name('index')),
            obj.pk, url_suffix, label)
    all_submissions_link.short_description = ''
    all_submissions_link.allow_tags = True

    def unprocessed_submissions_link(self, obj):
        return self.all_submissions_link(
            obj, _('See unprocessed submissions'),
            '&status=%s' % SubmissionStatusFilter.unprocessed_status)
    unprocessed_submissions_link.short_description = ''
    unprocessed_submissions_link.allow_tags = True

    def edit_link(self, obj):
        return '<a href="%s">%s</a>' % (
            reverse('wagtailadmin_pages:edit', args=(obj.pk,)),
            _('Edit this form page'))
    edit_link.short_description = ''
    edit_link.allow_tags = True


class SubmissionStatusFilter(SimpleListFilter):
    title = _('status')
    parameter_name = 'status'
    unprocessed_status = ','.join((SessionFormSubmission.COMPLETE,
                                   SessionFormSubmission.REVIEWED))

    def lookups(self, request, model_admin):
        yield (self.unprocessed_status, _('Complete or reviewed'))
        for status, verbose_status in SessionFormSubmission.STATUSES:
            if status != SessionFormSubmission.INCOMPLETE:
                yield status, verbose_status

    def queryset(self, request, queryset):
        status = self.value()
        if not status:
            return queryset
        if ',' in status:
            return queryset.filter(status__in=status.split(','))
        return queryset.filter(status=status)


class SubmissionPermissionHelper(PermissionHelper):
    def user_can_list(self, user):
        return get_forms_for_user(user).exists()

    def user_can_create(self, user):
        return False

    def user_can_edit_obj(self, user, obj):
        return False

    def user_can_inspect_obj(self, user, obj):
        return False

    def user_can_set_status_obj(self, user, obj):
        return user.can_set_status()


class SubmissionURLHelper(AdminURLHelper):
    def _get_action_url_pattern(self, action):
        if action == 'index':
            return r'^%s/%s/$' % (self.opts.app_label, 'submissions')
        return r'^%s/%s/%s/$' % (self.opts.app_label, 'submissions', action)

    def _get_object_specific_action_url_pattern(self, action):
        return r'^%s/%s/%s/(?P<instance_pk>[-\w]+)/$' % (
            self.opts.app_label, 'submissions', action)


class SubmissionButtonHelper(ButtonHelper):
    def set_status_button(self, pk, status, label, title, classnames_add=None,
                          classnames_exclude=None):
        if classnames_add is None:
            classnames_add = []
        if classnames_exclude is None:
            classnames_exclude = []
        classnames = self.finalise_classname(classnames_add,
                                             classnames_exclude)
        url = self.url_helper.get_action_url('set_status', quote(pk))
        url += '?status=' + status
        return {
            'url': url,
            'label': label,
            'classname': classnames,
            'title': title,
        }

    def reviewed_button(self, pk, classnames_add=None,
                        classnames_exclude=None):
        if classnames_add is None:
            classnames_add = []
        return self.set_status_button(pk, self.model.REVIEWED,
                                      _('mark as reviewed'),
                                      _('Mark this submission as reviewed'),
                                      classnames_add=classnames_add,
                                      classnames_exclude=classnames_exclude)

    def approve_button(self, pk, classnames_add=None,
                       classnames_exclude=None):
        if classnames_add is None:
            classnames_add = []
        if 'button-secondary' in classnames_add:
            classnames_add.remove('button-secondary')
        classnames_add = ['yes'] + classnames_add
        return self.set_status_button(pk, self.model.APPROVED, _('approve'),
                                      _('Approve this submission'),
                                      classnames_add=classnames_add,
                                      classnames_exclude=classnames_exclude)

    def reject_button(self, pk, classnames_add=None,
                      classnames_exclude=None):
        if classnames_add is None:
            classnames_add = []
        if 'button-secondary' in classnames_add:
            classnames_add.remove('button-secondary')
        classnames_add = ['no'] + classnames_add
        return self.set_status_button(pk, self.model.REJECTED, _('reject'),
                                      _('Reject this submission'),
                                      classnames_add=classnames_add,
                                      classnames_exclude=classnames_exclude)

    def get_buttons_for_obj(self, obj, exclude=None, classnames_add=None,
                            classnames_exclude=None):
        buttons = super().get_buttons_for_obj(
            obj, exclude=exclude, classnames_add=classnames_add,
            classnames_exclude=classnames_exclude)
        pk = getattr(obj, self.opts.pk.attname)
        status_buttons = []
        if obj.status != obj.REVIEWED:
            status_buttons.append(self.reviewed_button(
                pk, classnames_add=classnames_add,
                classnames_exclude=classnames_exclude))
        if obj.status != obj.APPROVED:
            status_buttons.append(self.approve_button(
                pk, classnames_add=classnames_add,
                classnames_exclude=classnames_exclude))
        if obj.status != obj.REJECTED:
            status_buttons.append(self.reject_button(
                pk, classnames_add=classnames_add,
                classnames_exclude=classnames_exclude))
        return status_buttons + buttons


class SetStatusView(InstanceSpecificView):
    def check_action_permitted(self, user):
        return self.permission_helper.user_can_set_status_obj(user,
                                                              self.instance)

    def get(self, request, *args, **kwargs):
        status = request.GET.get('status')
        if status in dict(self.model.STATUSES):
            previous_status = self.instance.status
            self.instance.status = status
            self.instance.save()
            verbose_label = self.instance.get_status_display()
            if 'revert' in request.GET:
                messages.success(request, 'Reverted to the “%s” status.'
                                 % verbose_label)
            else:
                revert_url = (self.url_helper.get_action_url('set_status',
                                                             self.instance_pk)
                              + '?revert&status=' + previous_status)
                messages.success(
                    request,
                    'Successfully changed the status to “%s”.' % verbose_label,
                    buttons=[messages.button(revert_url, _('Revert'))])
        url = request.META.get('HTTP_REFERER')
        if url is None:
            url = (self.url_helper.get_action_url('index')
                   + '?page_id=%s' % self.instance.page_id)
        return redirect(url)


class SubmissionAdmin(ModelAdmin):
    model = SessionFormSubmission
    menu_icon = 'form'
    permission_helper_class = SubmissionPermissionHelper
    url_helper_class = SubmissionURLHelper
    button_helper_class = SubmissionButtonHelper
    set_status_view_class = SetStatusView
    list_display = ('status', 'user', 'submit_time', 'last_modification')
    list_filter = (SubmissionStatusFilter, 'submit_time', 'last_modification')
    search_fields = ('user__first_name', 'user__last_name')

    def register_with_wagtail(self):
        @hooks.register('register_permissions')
        def register_permissions():
            return self.get_permissions_for_registration()

        @hooks.register('register_admin_urls')
        def register_admin_urls():
            return self.get_admin_urls_for_registration()

    def get_queryset(self, request):
        qs = super().get_queryset(request)
        form_pages = get_forms_for_user(request.user)
        return (qs.filter(page__in=form_pages)
                .exclude(status=self.model.INCOMPLETE))

    def get_form_page(self, request):
        form_pages = get_forms_for_user(request.user)
        try:
            return form_pages.get(pk=int(request.GET['page_id'])).specific
        except (KeyError, TypeError, ValueError, Page.DoesNotExist):
            pass

    # TODO: Find a cleaner way to display data from dynamic fields.
    def add_data_bridge(self, name, label):
        def data_bridge(obj):
            return obj.get_data().get(name)

        data_bridge.short_description = label
        setattr(self, name, data_bridge)

    def get_list_display(self, request):
        form_page = self.get_form_page(request)
        if form_page is None:
            return self.list_display
        fields = []
        for name, label in form_page.get_data_fields():
            fields.append(name)
            self.add_data_bridge(name, label)
        return fields

    def set_status_view(self, request, instance_pk):
        kwargs = {'model_admin': self, 'instance_pk': instance_pk}
        view_class = self.set_status_view_class
        return view_class.as_view(**kwargs)(request)

    def get_admin_urls_for_registration(self):
        urls = super().get_admin_urls_for_registration()
        urls += (
            url(self.url_helper.get_action_url_pattern('set_status'),
                self.set_status_view,
                name=self.url_helper.get_action_url_name('set_status')),
        )
        return urls


# @hooks.register('construct_main_menu')
# def hide_old_forms_module(request, menu_items):
#     from wagtail.contrib.forms.wagtail_hooks import FormsMenuItem
#     for menu_item in menu_items:
#         if isinstance(menu_item, FormsMenuItem):
#             menu_items.remove(menu_item)

# modeladmin_register(FormAdmin)
# modeladmin_register(SubmissionAdmin)



# Generated by Django 2.1.5 on 2019-03-19 23:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coderedcms', '0009_auto_20190201_1546'),
    ]

    operations = [
        migrations.AlterField(
            model_name='generalsettings',
            name='from_email_address',
            field=models.CharField(blank=True, help_text='The default email address this site appears to send from. For example: "sender@example.com" or "Sender Name <sender@example.com>" (without quotes)', max_length=255, verbose_name='From email address'),
        ),
    ]



"""
Enhancements to wagtail.contrib.forms.
"""
import csv
import os
import re
from django import forms
from django.core.exceptions import ValidationError
from django.db import models
from django.http import HttpResponse
from django.utils.translation import gettext_lazy as _
from wagtail.contrib.forms.views import SubmissionsListView as WagtailSubmissionsListView
from wagtail.contrib.forms.forms import FormBuilder
from wagtail.contrib.forms.models import AbstractFormField

from coderedcms.settings import cr_settings
from coderedcms.utils import attempt_protected_media_value_conversion

FORM_FIELD_CHOICES = (
    (_("Text"), (
        ("singleline", _("Single line text")),
        ("multiline", _("Multi-line text")),
        ("email", _("Email")),
        ("number", _("Number - only allows integers")),
        ("url", _("URL")),
    ),),
    (_("Choice"), (
        ("checkboxes", _("Checkboxes")),
        ("dropdown", _("Drop down")),
        ("radio", _("Radio buttons")),
        ("multiselect", _("Multiple select")),
        ("checkbox", _("Single checkbox")),
    ),),
    (_("Date & Time"), (
        ("date", _("Date")),
        ("time", _("Time")),
        ("datetime", _("Date and time")),
    ),),
    (_("File Upload"), (
        ("file", _("Secure File - login required to access uploaded files")),
    ),),
    (_("Other"), (
        ("hidden", _("Hidden field")),
    ),),
)


# Files

class SecureFileField(forms.FileField):
    custom_error_messages = {
        'blacklist_file': _('Submitted file is not allowed.'),
        'whitelist_file': _('Submitted file is not allowed.')
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.error_messages.update(self.custom_error_messages)

    def validate(self, value):
        super(SecureFileField, self).validate(value)
        if value:
            self._check_whitelist(value)
            self._check_blacklist(value)

    def _check_whitelist(self, value):
        if cr_settings['PROTECTED_MEDIA_UPLOAD_WHITELIST']:
            if os.path.splitext(value.name)[1].lower() not in cr_settings['PROTECTED_MEDIA_UPLOAD_WHITELIST']:  # noqa
                raise ValidationError(self.error_messages['whitelist_file'])

    def _check_blacklist(self, value):
        if cr_settings['PROTECTED_MEDIA_UPLOAD_BLACKLIST']:
            if os.path.splitext(value.name)[1].lower() in cr_settings['PROTECTED_MEDIA_UPLOAD_BLACKLIST']:  # noqa
                raise ValidationError(self.error_messages['blacklist_file'])


# Date

class CoderedDateInput(forms.DateInput):
    template_name = 'coderedcms/formfields/date.html'


class CoderedDateField(forms.DateField):
    widget = CoderedDateInput()


# Datetime

class CoderedDateTimeInput(forms.DateTimeInput):
    template_name = 'coderedcms/formfields/datetime.html'


class CoderedDateTimeField(forms.DateTimeField):
    widget = CoderedDateTimeInput()
    input_formats = ['%Y-%m-%dT%H:%M', '%m/%d/%Y %I:%M %p', '%m/%d/%Y %I:%M%p', '%m/%d/%Y %H:%M']


# Time

class CoderedTimeInput(forms.TimeInput):
    template_name = 'coderedcms/formfields/time.html'


class CoderedTimeField(forms.TimeField):
    widget = CoderedTimeInput()
    input_formats = ['%H:%M', '%I:%M %p', '%I:%M%p']


class CoderedFormBuilder(FormBuilder):
    """
    Enhance wagtail FormBuilder with additional custom fields.
    """

    def create_file_field(self, field, options):
        return SecureFileField(**options)

    def create_date_field(self, field, options):
        return CoderedDateField(**options)

    def create_datetime_field(self, field, options):
        return CoderedDateTimeField(**options)

    def create_time_field(self, field, options):
        return CoderedTimeField(**options)


class CoderedSubmissionsListView(WagtailSubmissionsListView):
    def get_csv_response(self, context):
        filename = self.get_csv_filename()
        response = HttpResponse(content_type='text/csv; charset=utf-8')
        response['Content-Disposition'] = 'attachment;filename={}'.format(filename)

        writer = csv.writer(response)
        writer.writerow(context['data_headings'])
        for data_row in context['data_rows']:
            modified_data_row = []
            for cell in data_row:
                modified_cell = attempt_protected_media_value_conversion(self.request, cell)
                modified_data_row.append(modified_cell)

            writer.writerow(modified_data_row)
        return response


class CoderedFormField(AbstractFormField):
    class Meta:
        abstract = True

    field_type = models.CharField(
        verbose_name=_('field type'),
        max_length=16,
        choices=FORM_FIELD_CHOICES,
        blank=False,
        default='Single line text'
    )


class SearchForm(forms.Form):
    s = forms.CharField(
        max_length=255,
        required=False,
        label=_('Search'),
    )
    t = forms.CharField(
        widget=forms.HiddenInput,
        max_length=255,
        required=False,
        label=_('Page type'),
    )


def get_page_model_choices():
    """
    Returns a list of tuples of all creatable Codered pages
    in the format of ("Custom Codered Page", "CustomCoderedPage")
    """
    from coderedcms.models import get_page_models
    return (
        (
            page.__name__,
            re.sub(
                r'((?<=[a-z])[A-Z]|(?<!\A)[A-Z](?=[a-z]))',
                r' \1',
                page.__name__
            )
        ) for page in get_page_models() if page.is_creatable
    )
